--
-- Measure Helper
--
-- @author  	V!rus(somersault[at]ich-habe-fertig.com)
-- @version 	v2
-- @date        08/01/2019 (dd/mm/yyyy)
--
-- This mod helps to measure the distance between a point and the players position
-- After the first use a XML-File with some aditional settings is available in the mod folder
--

--
-- You are free to modify this work.
-- If you modify and publish this work, or distribute it, please do without any monetization and include all authors.
-- This mod is published in it's current state on modhoster.de
--

--
-- CHANGELOG
-- v2.0 08/01/2019(dd/mm/yyyy)
-- + ConfigXML in mod folder: Change color, and toggle gui-elements
-- + select between three units of measurement (m/ft/yd) - Idea and own version for FS19 from kenny456
-- + keybinds can be configured by the user ingame
-- * bugfix in the calculation (y-axis) - thanks to WX for the notice
--

--
-- Measure
--
Measure = {
    active = false,
    units = {"m", "ft", "yd"}, --[1]="m", ...
    configXml = nil,
    config = {
        unit = 0, --0:m 1:ft 2:yd
        showStartingPoint = true,
        showDistanceInHelp = true,
        showDistanceOnScreen = true,
        showKeybindHelp = true,
        fontR = 0,
        fontG = 0,
        fontB = 0
    },
    configMinMax = {
        unit = {0,2, math.floor},
        fontR = {0,1},
        fontG = {0,1},
        fontB = {0,1}
    }
}


--
-- Measure.prerequisitesPresent
--
function Measure:prerequisitesPresent(specializations)
    return true;
end


--
-- Measure:loadMap
--
function Measure:loadMap(name)
	Measure.active = false;
    local configFile = g_modsDirectory .. "/MeasureConfig.xml";
    if fileExists(configFile) then
        Measure.configXml = loadXMLFile("Measure_XML", configFile);
        Measure:loadFromXml();
    else
        Measure.configXml = createXMLFile("Measure_XML", configFile, "MeasureConfig");
        Measure:printHelpToXML();
        Measure:saveToXml();
    end
end

--
-- Measure:loadFromXml
--
function Measure:loadFromXml()
    if Measure.configXml == nil then
        return;
    end

    local v = nil;
    for key,value in pairs(Measure.config) do
        if type(Measure.config[key]) == "boolean" then
            v = getXMLBool(Measure.configXml, "MeasureConfig." .. key);
        end
        if type(Measure.config[key]) == "number" then
            v = getXMLFloat(Measure.configXml, "MeasureConfig." .. key);
            v = Measure:checkFloatsFromXML(key, v);
        end
        if type(Measure.config[key]) == "string" then
            --v = getXMLString(Measure.configXml, "MeasureConfig." .. key);
        end

        if not (v == nil) then
            Measure.config[key] = v;
        end
    end

    -- in case some values where not in the xml or assinged to fucked up values...
    Measure:saveToXml();
end

--
-- Measure:checkFloatsFromXML
--
function Measure:checkFloatsFromXML(key, value)
    if value == nil or not type(value) == "number" then
        return nil;
    end
    local min = Measure.configMinMax[key][1];
    local max = Measure.configMinMax[key][2];
    if min == nil or max == nil then
        return nil;
    end
    if (value < min) or (value > max) then
        print("MeasureHelp: Value(" .. value .. ") of " .. key .. " is out of range: " .. min .. "-" .. max);
        return nil;
    else
        if (not (Measure.configMinMax[key][3] == nil)) and (type(Measure.configMinMax[key][3]) == "function") then
            return Measure.configMinMax[key][3](value);
        else
            return value;
        end
    end
end

--
-- Measure:printHelpToXML
--
function Measure:printHelpToXML()
    if Measure.configXml == nil then
        return;
    end
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP", "Here is some help, to know whats going on and how to set values in this file:");
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP.UNIT", "unit: 0 for meter, 1 for feet, 2 for yard");
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP.FONT", "fontX: 0.0 - 1.0 for the R(ed), G(reen) or B(lue) part of the color for the font on screen");
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP.SHOWX", "showX: true or false to show or hide what is described by X");
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP.MistakesWereMade", "Nothing can go wrong. In case you manage to f*ck it up anyway, just delete this file. After loading a game(map), there will be a new one for you to play with.");
    setXMLString(Measure.configXml, "MeasureConfig.XMLHELP.ChangesDontWork", "Please mind: A reload of the savegame is required to see your changes in action.");
end  

--
-- Measure:saveToXml
--
function Measure:saveToXml()
    if Measure.configXml == nil then
        return;
    end
    for key,value in pairs(Measure.config) do
        if type(Measure.config[key]) == "boolean" then
            setXMLBool(Measure.configXml, "MeasureConfig." .. key, value);
        end
        if type(Measure.config[key]) == "number" then
            setXMLFloat(Measure.configXml, "MeasureConfig." .. key, value);
        end
        if type(Measure.config[key]) == "string" then
            setXMLString(Measure.configXml, "MeasureConfig." .. key, value);
        end
    end
    saveXMLFile(Measure.configXml);
end

--
-- Measure:deleteMap
--
function Measure:deleteMap()
end

--
--  Measure:mouseEvent
--
function Measure:mouseEvent(posX, posY, isDown, isUp, button)
end

--
-- Measure:keyEvent
--
function Measure:keyEvent(unicode, sym, modifier, isDown)
end

--
-- Measure:setStartingPoint
--
function Measure:setStartingPoint()
    local p = nil;
    -- player in vehicle?
    if g_currentMission.player ~= nil then
       p = g_currentMission.player.rootNode;
    end
    if g_currentMission.controlledVehicle ~= nil then
        p = g_currentMission.controlledVehicle.rootNode;
    end
    -- player exists?
    if p ~= nil then
        -- get player cords
        aX, aY, aZ = getWorldTranslation(p);
        Measure.active = true;
    else
        -- not possible to find player position
        print("Not a player and not in vehicle!? => MeasureHelp will not be activated");
    end
end

--
-- Measure:update
--
function Measure:update(dt)

    if InputBinding.hasEvent(InputBinding.Measure_StartingPoint_Toggle) then
        if Measure.active then
            Measure.active = false;
        else
            Measure:setStartingPoint();
        end
    end

    -- active?
    if Measure.active then
        if Measure.config.showKeybindHelp then
            g_currentMission:addHelpButtonText(g_i18n:getText("Measure_StartingPoint_Delete"), InputBinding.Measure_StartingPoint_Toggle, nil, GS_PRIO_LOW);
            g_currentMission:addHelpButtonText(g_i18n:getText("Measure_Unit_Toggle"), InputBinding.Measure_Unit_Toggle, nil, GS_PRIO_LOW);
        end

        if InputBinding.hasEvent(InputBinding.Measure_Unit_Toggle) then
            Measure.config.unit = (Measure.config.unit + 1) % 3;
            Measure:saveToXml();
        end

        -- player in vehicle?
        local p = nil;
        if g_currentMission.player ~= nil then
            p = g_currentMission.player.rootNode;
        end
        if g_currentMission.controlledVehicle ~= nil then
            p = g_currentMission.controlledVehicle.rootNode;
        end

        -- player exists?
        if p ~= nil then
            -- get playerCords and calculate the distance
            bX, bY, bZ = getWorldTranslation(p);
            local c2 = ( math.pow( (aX - bX), 2 ) + math.pow( (aZ - bZ), 2 ) );
            --m
            distance = math.sqrt( ( c2 + math.pow( (aY - bY), 2 ) ) );
            --ft
            if Measure.config.unit == 1 then
                distance = distance * 3.28083
            end
            --yd
            if Measure.config.unit == 2 then
                distance = distance * 1.09361
            end
            --2 digit precision
            distance = math.floor(distance * 100) / 100;

            if Measure.config.showDistanceInHelp then
                g_currentMission:addExtraPrintText(distance .. Measure.units[(Measure.config.unit + 1)]);
            end
        else
            Measure.active = false;
            print("Not a player and not in vehicle!? => MeasureHelp set active to false");
        end
    else
        if Measure.config.showKeybindHelp then
            g_currentMission:addHelpButtonText(g_i18n:getText("Measure_StartingPoint_Set"), InputBinding.Measure_StartingPoint_Toggle, nil, GS_PRIO_LOW);
        end
    end
end

--
-- Measure:draw
--
function Measure:draw()

    if Measure.active then
        if Measure.config.showStartingPoint then
            -- draw yellow and black pole
            local c = 0;
            for y = (aY-1), (aY+2), 0.2
            do
                drawDebugLine(aX, y, aZ, c, c, 0, aX, y + 0.2, aZ, c, c, 0);
                if c == 0 then
                    c = 1;
                else
                    c = 0;
                end
            end
            -- red x
            drawDebugLine(aX-0.1, aY, aZ, 1, 0, 0, aX+0.1, aY, aZ, 1, 0, 0);
            drawDebugLine(aX, aY, aZ-0.1, 1, 0, 0, aX, aY, aZ+0.1, 1, 0, 0);
        end

        if Measure.config.showDistanceOnScreen then
            -- draw the text
            setTextColor(Measure.config.fontR, Measure.config.fontG, Measure.config.fontB, 1);   
            setTextBold(true);
            setTextAlignment(RenderText.ALIGN_CENTER);
            renderText(0.5, 0.1, 0.05, distance .. Measure.units[(Measure.config.unit + 1)]);
        end

    end
end

--
-- Register Mod Events
--
addModEventListener(Measure);